/* 
   DS18x20 Demo-Program
   
   V 0.9.2, 2/2011
   
   by Martin Thomas <eversmith@heizung-thomas.de>
   http://www.siwawi.arubi.uni-kl.de/avr-projects
    
   features:
   - DS18X20 and 1-Wire code is based on an example from Peter 
     Dannegger
   - uses Peter Fleury's uart-library which is very portable 
   - additional functions not found in the  uart-lib available
     in uart.h/.c
   - CRC-check based on code from Colin O'Flynn
   - accesses multiple sensors on multiple 1-Wire busses
   - example how to address every sensor in the bus by ROM-code
   - independant of system-timers (more portable) but some
     (very short) delays used
   - avr-libc's stdint.h in use 
   - no central include-file, parts of the code can be used as
     "library" easily
   - verbose output example
   - one-wire-bus can be changed at runtime if OW_ONE_BUS
     is not defined in onewire.h. There are still minor timing 
     issues when using the dynamic bus-mode
   - example on read/write of DS18x20 internal EEPROM
*/
/* This example has been tested with ATmega324P at 3.6864MHz and 16Mhz */
#include <avr/version.h>
#if __AVR_LIBC_VERSION__ < 10606UL
#error "please update to avrlibc 1.6.6 or newer, not tested with older versions"
#endif
#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <string.h>
#include <stdint.h>
#include <stdio.h>
//#include "uart_addon.h"
#include "onewire.h"
#include "ds18x20.h"
#include "sensor.h"
#include "../usart/usart.h"
#include "../config.h"
#include "../s_tools__.h"
#include "crc8.h"
//uint8_t gSensorIDs[MAXSENSORS][OW_ROMCODE_SIZE];
uint8_t diff, nSensors;
uint8_t gSensorIDs[MAXSENSORS][OW_ROMCODE_SIZE];
char sensor_id_str[MAXSENSORS][SENSORDATA_LEN];


char u5_vorrotbalken[4];
char u5_gruenbalken[4];
char u5_nachrotbalken[4];


extern volatile uint8_t error;
extern unsigned char erststart;
void clr_all_sensors(void)
{	unsigned char i;
	unsigned char j;
	usart_write("lsche alle Temp Sensoren \r\n");
	for(i=0; i<MAXSENSORS; i++)
		for(j=0; j<SENSORDATA_LEN;j++)
			sensor_id_str[i][j]=0;
	usart_write("alle Temp Sensoren gelscht \r\n");
}




void show_all_sensors(void)
{	unsigned char i;
	unsigned char j;
	usart_write("zeige alle Temp Sensoren \r\n");
	for(i=0; i<MAXSENSORS; i++)
	{	for(j=0; j<OW_ROMCODE_SIZE; j++)
			usart_write("%2x", (unsigned int)sensor_id_str[i][j]);
		if ( sensor_id_str[i][0] == DS18S20_FAMILY_CODE  || sensor_id_str[i][0] == DS18B20_FAMILY_CODE || sensor_id_str[i][0] == DS1822_FAMILY_CODE )
			switch(i)

			{

				case WZ:

					usart_write(" %sC, Wohnzimmer,     Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case WZ_WW:

					usart_write(" %sC, Wohnzimmer warm,Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case SZ:

					usart_write(" %sC, Schlafzimmer,   Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case KZ:

					usart_write(" %sC, kl. Zimmer,     Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case ZS:

					usart_write(" %sC, Zhlerschrank,  Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case BK:

					usart_write(" %sC, Balkon,         Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				default:			

					break;

			}

		else
			usart_write("\r\n");
	}
}


void read_all_sensors(void)
{	unsigned char i;
	unsigned char j;
	uint8_t bus;

	usart_write("zeige alle Temperaturen \r\n");
	for(i=0; i<MAXSENSORS; i++)
	{	for(j=0; j<OW_ROMCODE_SIZE; j++)
			usart_write("%2x", (unsigned int)sensor_id_str[i][j]);
		if ( sensor_id_str[i][0] == DS18S20_FAMILY_CODE  || sensor_id_str[i][0] == DS18B20_FAMILY_CODE || sensor_id_str[i][0] == DS1822_FAMILY_CODE )
		{	

			bus=sensor_id_str[i][BUS_BEGIN]-'0';
			ein_sensor_lesen_komma_einsetzen( i, (uint8_t *)&sensor_id_str[i][0], bus );

			switch(i)

			{

				case WZ:

					usart_write(" %sC, Wohnzimmer,     Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case WZ_WW:

					usart_write(" %sC, Wohnzimmer warm,Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case SZ:

					usart_write(" %sC, Schlafzimmer,   Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case KZ:

					usart_write(" %sC, kl. Zimmer,     Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case ZS:

					usart_write(" %sC, Zhlerschrank,  Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				case BK:

					usart_write(" %sC, Balkon,         Bus:#%s, N:%s, M:%s, P:%s  \r\n", (char *)&sensor_id_str[i][TEMP_BEGIN], (char *)&sensor_id_str[i][BUS_BEGIN], (char *)&sensor_id_str[i][NULL_BEGIN], (char *)&sensor_id_str[i][MINUS_BEGIN], (char *)&sensor_id_str[i][PLUS_BEGIN]);
					break;

				default:			

					break;

			}

		}

		else
			usart_write("\r\n");
	}
}


//jar_debug
//char *ein_sensor_lesen_komma_einsetzen(uint8_t, uint8_t *, uint8_t );
//char *ein_sensor_lesen_komma_einsetzen(uint8_t wer, uint8_t *id, uint8_t bus )
void ein_sensor_lesen_komma_einsetzen(uint8_t wer, uint8_t *id, uint8_t bus )
{	
//jar_debug
//char *ein_sensor_lesen_komma_einsetzen(uint8_t, uint8_t *, uint8_t );
//	static char back[10];
		
	int16_t 	decicelsius;
	int8_t 	tmp_temp, retry=0;			
	char tmp_buffer[10], tmp_buffer2[10];
	char *komma_ptr=0;

//jar_debug
//char *ein_sensor_lesen_komma_einsetzen(uint8_t, uint8_t *, uint8_t );
//	strcpy(back, "nix");

#ifndef OW_ONE_BUS
	switch(bus)
	{	case 1:
			ow_set_bus(&W1_IN,&W1_OUT,&W1_DDR,W1_PIN);_delay_ms(1);
			//bus1;
			break;
		case 2:
			ow_set_bus(&W2_IN,&W2_OUT,&W2_DDR,W2_PIN);_delay_ms(1);
			//bus2;
			break;
		case 3:
			ow_set_bus(&W3_IN,&W3_OUT,&W3_DDR,W3_PIN);_delay_ms(1);
			//bus3;
			break;
		case 4:
			ow_set_bus(&W4_IN,&W4_OUT,&W4_DDR,W4_PIN);_delay_ms(1);
			//bus4;
			break;
		case 5:
			ow_set_bus(&W5_IN,&W5_OUT,&W5_DDR,W5_PIN);_delay_ms(1);
			//bus5;
			break;
		default:
			break;
	}
#endif

	if ( DS18X20_start_meas( DS18X20_POWER_PARASITE, NULL ) == DS18X20_OK) 
	{	_delay_ms( DS18B20_TCONV_12BIT );
		do

		{

			if(DS18X20_read_decicelsius( (uint8_t *)&sensor_id_str[wer][0], &decicelsius ) == DS18X20_OK ) 
			{	if(retry)


usart_write("retry %i retry DS18X20_read_decicelsius in sensor.c Ln 181 \r\n", retry );
				

				retry=3;

				if(decicelsius<0)
					strcpy( (char *)&sensor_id_str[wer][TEMP_BEGIN], insert_char(trimm_string(' ', ltoa( decicelsius,tmp_buffer,10), 4), ',', 3) );
				else
				{	strcpy( tmp_buffer2, "+");
					strcat( tmp_buffer2, ltoa( decicelsius,tmp_buffer,10));
			
//jar_debug
//char *ein_sensor_lesen_komma_einsetzen(uint8_t, uint8_t *, uint8_t );
//					strcpy( back, insert_char(trimm_string(' ', tmp_buffer2, 4), ',', 3) );
			
//jar_debug
//char *ein_sensor_lesen_komma_einsetzen(uint8_t, uint8_t *, uint8_t );
//if(wer<MAXSENSORS)
					strcpy( (char *)&sensor_id_str[wer][TEMP_BEGIN], insert_char(trimm_string(' ', tmp_buffer2, 4), ',', 3) );

				}

//jar_debug
//char *ein_sensor_lesen_komma_einsetzen(uint8_t, uint8_t *, uint8_t );
//			if(wer<MAXSENSORS)
//			{
				komma_ptr=strchr((char *)&sensor_id_str[wer][TEMP_BEGIN], ',');
				komma_ptr--;
				if( *komma_ptr =='+' || *komma_ptr =='-' )
				{	*(komma_ptr-1) = *komma_ptr;
					*(komma_ptr) = '0';
				}
				strcpy( (char *)&sensor_id_str[wer][BUS_BEGIN], itoa(bus,tmp_buffer,10) );

				//Balken
				if(strstr((char *)&sensor_id_str[wer][TEMP_BEGIN], "-"))
					strcpy(tmp_buffer, strstr((char *)&sensor_id_str[wer][TEMP_BEGIN], "-"));
				if(strstr((char *)&sensor_id_str[wer][TEMP_BEGIN], "+"))
					strcpy(tmp_buffer, strstr((char *)&sensor_id_str[wer][TEMP_BEGIN], "+"));

				tmp_temp = atoi(tmp_buffer);

				if(tmp_temp<0) 
				{	//NULL_BEGIN
					if( tmp_temp<-54 )
						strcpy((char *)&sensor_id_str[wer][NULL_BEGIN], itoa(0, tmp_buffer, 10));
					else
						if( tmp_temp<0 )
							strcpy((char *)&sensor_id_str[wer][NULL_BEGIN], itoa(140+(4*tmp_temp), tmp_buffer, 10) );
				}
				else
					strcpy((char *)&sensor_id_str[wer][NULL_BEGIN], itoa(136, tmp_buffer, 10) );

				//MINUS_BEGIN
				if(tmp_temp<0) 
				{	if( tmp_temp<-35 )


						strcpy((char *)&sensor_id_str[wer][MINUS_BEGIN], itoa(140, tmp_buffer, 10));
					else


						if( tmp_temp<0 )


							strcpy((char *)&sensor_id_str[wer][MINUS_BEGIN], itoa((-4*tmp_temp), tmp_buffer, 10) );


						else


							strcpy((char *)&sensor_id_str[wer][MINUS_BEGIN], itoa(4, tmp_buffer, 10) );


				}


				else


					strcpy((char *)&sensor_id_str[wer][MINUS_BEGIN], itoa(4, tmp_buffer, 10) );



				//PLUS_BEGIN
				if(tmp_temp<0) 
					strcpy((char *)&sensor_id_str[wer][PLUS_BEGIN], itoa(4, tmp_buffer, 10));
				else
					if( tmp_temp<2 )
						strcpy((char *)&sensor_id_str[wer][PLUS_BEGIN], itoa(4, tmp_buffer, 10) );
					else
						if( tmp_temp<51 )
							strcpy((char *)&sensor_id_str[wer][PLUS_BEGIN], itoa((4*tmp_temp), tmp_buffer, 10) );
						else
							strcpy((char *)&sensor_id_str[wer][PLUS_BEGIN], itoa(200, tmp_buffer, 10) );

			} // if(DS18X20_read_decicelsius( (uint8_t *)&sensor_id_str[wer][0], &decicelsius ) == DS18X20_OK ) 
			else 
			{	strcpy( (char *)&sensor_id_str[wer][TEMP_BEGIN], "Error");
				error++;

				retry++;
			} // if !(DS18X20_read_decicelsius( (uint8_t *)&sensor_id_str[wer][0], &decicelsius ) == DS18X20_OK ) 
//jar_debug
//char *ein_sensor_lesen_komma_einsetzen(uint8_t, uint8_t *, uint8_t );
//		}
		//*(&sensor_id_str[(a+nSensors)][13])=0; 
		}while(retry<3);

	}
	else 
	{	if(erststart)
			usart_write("Start meas. failed (short circuit?)\r\n" );
		error++;
	}
//jar_debug
//char *ein_sensor_lesen_komma_einsetzen(uint8_t, uint8_t *, uint8_t );
//	return(back);
}


uint8_t search_sensors(uint8_t bus)
{	uint8_t i=0, diff=0, nSensors=0, a=0;
	uint8_t id[OW_ROMCODE_SIZE];
	char tmp_buffer[10];

#ifndef OW_ONE_BUS
	switch(bus)
	{	case 1:
			ow_set_bus(&W1_IN,&W1_OUT,&W1_DDR,W1_PIN);_delay_ms(1);
			//bus1;
			break;
		case 2:
			ow_set_bus(&W2_IN,&W2_OUT,&W2_DDR,W2_PIN);_delay_ms(1);
			//bus2;
			break;
		case 3:
			ow_set_bus(&W3_IN,&W3_OUT,&W3_DDR,W3_PIN);_delay_ms(1);
			//bus3;
			break;
		case 4:
			ow_set_bus(&W4_IN,&W4_OUT,&W4_DDR,W4_PIN);_delay_ms(1);
			//bus4;
			break;
		case 5:
			ow_set_bus(&W5_IN,&W5_OUT,&W5_DDR,W5_PIN);_delay_ms(1);
			//bus5;
			break;
		default:
			break;
	}
#endif
	
//usart_write("schon gefunden Sensorsumme %i \r\n", a);
	ow_reset();
	if(erststart)
		usart_write("suche Sensor(en) auf Bus: #%i ", bus);
	diff = OW_SEARCH_FIRST;
	while ( diff != OW_LAST_DEVICE && (a) < MAXSENSORS ) 
	{	DS18X20_find_sensor( &diff, &id[0] );
		if( diff == OW_PRESENCE_ERR ) 
		{	if(erststart)
				usart_write("\rNo Sensor found");
			break;
		}
		if( diff == OW_DATA_ERR ) 
		{	if(erststart)
				usart_write("\rBus Error\r\n");
			break;
		}
		for ( i=0; i < OW_ROMCODE_SIZE; i++ )
		{	if ( i == 0 ) 
			{	if ( id[0] == DS18S20_FAMILY_CODE  || id[0] == DS18B20_FAMILY_CODE || id[0] == DS1822_FAMILY_CODE )
				{	if ( !crc8( id, OW_ROMCODE_SIZE) )
					{	sprintf(tmp_buffer,"%02x%02x%02x%02x", id[1], id[2], id[3], id[4]);
						usart_write("%s\r\n", tmp_buffer);
#ifdef WOHNZIMMER
						if(strstr(WOHNZIMMER,tmp_buffer))
						{	usart_write("Wohnzimmer gefunden\r\n");
							a=WZ;
							memcpy((uint8_t *)&sensor_id_str[a][0],(uint8_t *)&id[0],OW_ROMCODE_SIZE);
							ein_sensor_lesen_komma_einsetzen(a, id, bus);
							nSensors++;
						}
						else 
#endif
#ifdef WOHNZIMMER_WW
						if(strstr(WOHNZIMMER_WW,tmp_buffer))
						{	usart_write("Wohnzimmer warme Wand gefunden\r\n");
							a=WZ_WW;
							memcpy((uint8_t *)&sensor_id_str[a][0],(uint8_t *)&id[0],OW_ROMCODE_SIZE);
							ein_sensor_lesen_komma_einsetzen(a, id, bus);
							nSensors++;
						}
						else 
#endif
#ifdef BALKON
						if(strstr(BALKON,tmp_buffer))
						{	usart_write("Balkon gefunden\r\n");
							a=BK;
							memcpy((uint8_t *)&sensor_id_str[a][0],(uint8_t *)&id[0],OW_ROMCODE_SIZE);
							ein_sensor_lesen_komma_einsetzen(a, id, bus);
							nSensors++;
						}
						else 
#endif
#ifdef SCHLAFZIMMER
							if(strstr(SCHLAFZIMMER,tmp_buffer))
							{	usart_write("Schlafzimmer gefunden\r\n");
								a=SZ;
								memcpy((uint8_t *)&sensor_id_str[a][0],(uint8_t *)&id[0],OW_ROMCODE_SIZE);
								ein_sensor_lesen_komma_einsetzen(a, id, bus);
								nSensors++;
							}
							else 
#endif
#ifdef KLEINESZIMMER
								if(strstr(KLEINESZIMMER,tmp_buffer))
								{	usart_write("kleines Zimmer gefunden\r\n");
									a=KZ;
									memcpy((uint8_t *)&sensor_id_str[a][0],(uint8_t *)&id[0],OW_ROMCODE_SIZE);
									ein_sensor_lesen_komma_einsetzen(a, id, bus);
									nSensors++;
								}
								else 
#endif
#ifdef ZAEHLERSCHRANK
									if(strstr(ZAEHLERSCHRANK,tmp_buffer))
									{	usart_write("Zhlerschrank gefunden\r\n");
										a=ZS;
										memcpy((uint8_t *)&sensor_id_str[a][0],(uint8_t *)&id[0],OW_ROMCODE_SIZE);
										ein_sensor_lesen_komma_einsetzen(a, id, bus);
										nSensors++;
									}
									else
#endif
									{	for ( i=0; i < OW_ROMCODE_SIZE; i++ )
											usart_write("%2x", id[i]);
										usart_write("\r\n");
										//usart_write("unbekannter Sensor STOP in sensor.c line390 \r\n");
//jar_debug
//char *ein_sensor_lesen_komma_einsetzen(uint8_t, uint8_t *, uint8_t );
//										usart_write("unbekannter Sensor: %s\r\n",ein_sensor_lesen_komma_einsetzen(9, id, bus));
										usart_write("unbekannter Sensor! \r\n");
										//while(1);
									}
					}
				}
			}
		}
	} // while ( diff != OW_LAST_DEVICE && (a+nSensors) < MAXSENSORS )
	
	if(erststart)
		usart_write("%i Sensor(en) gefunden \r\n", nSensors);
	return nSensors;
}


void read_found_single_sensor(uint8_t bus, unsigned long sensor_id)
{	uint8_t i=0, diff=0, a=0, retry=0;
	uint8_t id[OW_ROMCODE_SIZE];
	int16_t decicelsius;
	char tmp_buffer[10]; char tmp_buffer2[10];
#ifndef OW_ONE_BUS
	switch(bus)
	{	case 1:
			ow_set_bus(&W1_IN,&W1_OUT,&W1_DDR,W1_PIN);_delay_ms(1);
			//bus1;
			break;
		case 2:
			ow_set_bus(&W2_IN,&W2_OUT,&W2_DDR,W2_PIN);_delay_ms(1);
			//bus2;
			break;
		case 3:
			ow_set_bus(&W3_IN,&W3_OUT,&W3_DDR,W3_PIN);_delay_ms(1);
			//bus3;
			break;
		case 4:
			ow_set_bus(&W4_IN,&W4_OUT,&W4_DDR,W4_PIN);_delay_ms(1);
			//bus4;
			break;
		case 5:
			ow_set_bus(&W5_IN,&W5_OUT,&W5_DDR,W5_PIN);_delay_ms(1);
			//bus5;
			break;
		default:
			break;
	}
#endif
	
	ow_reset();
	diff = OW_SEARCH_FIRST;
	while ( diff != OW_LAST_DEVICE ) 
	{	DS18X20_find_sensor( &diff, &id[0] );
		if( diff == OW_PRESENCE_ERR ) 
			break;
		if( diff == OW_DATA_ERR ) 
		{	usart_write("Bus Error in read_sensor()\r\n");
			break;
		}
//-----------------------------------------------------------------
		if ( DS18X20_start_meas( DS18X20_POWER_PARASITE, NULL ) == DS18X20_OK) 
		{	_delay_ms( DS18B20_TCONV_12BIT );
			do
			{
				if(DS18X20_read_decicelsius( (uint8_t *)&id[0], &decicelsius ) == DS18X20_OK ) 
				{	if(retry)

usart_write("retry %i retry DS18X20_read_decicelsius in sensor.c Ln 421 \r\n", retry );
					for(i=0; i<a; i++)
					{	
						if(!memcmp((uint8_t *)&sensor_id_str[i][0], (uint8_t *)&id[0], OW_ROMCODE_SIZE))
						{	if(decicelsius<0)
								strcpy( (char *)&sensor_id_str[i][OW_ROMCODE_SIZE], insert_char(trimm_string(' ', ltoa( decicelsius,tmp_buffer,10), 4), ',', 3) );
							else
							{	strcpy( tmp_buffer2, "+");
								strcat( tmp_buffer2, ltoa( decicelsius,tmp_buffer,10));
								strcpy( (char *)&sensor_id_str[i][OW_ROMCODE_SIZE], insert_char(trimm_string(' ', tmp_buffer2, 4), ',', 3) );
							}
							if( *(char *)&sensor_id_str[i][OW_ROMCODE_SIZE+2]=='+' || *(char *)&sensor_id_str[i][OW_ROMCODE_SIZE+2]=='-' )
							{	*(char *)&sensor_id_str[i][OW_ROMCODE_SIZE+1]=*(char *)&sensor_id_str[i][OW_ROMCODE_SIZE+2];
								*(char *)&sensor_id_str[i][OW_ROMCODE_SIZE+2]='0';
							}
	//usart_write(" %s C \r\n", (char *)&sensor_id_str[i][OW_ROMCODE_SIZE]);
						} // if(!memcmp((uint8_t *)&sensor_id_str[i][0], (uint8_t *)&id[0], OW_ROMCODE_SIZE))
					} // for(i=0; i<a; i++)
				} // for(i=0; i<a; i++)
				else 
				{	strcpy( (char *)&sensor_id_str[(nSensors)][OW_ROMCODE_SIZE], "Error");
					retry++;
					error++;
				} // if !(DS18X20_read_decicelsius( (uint8_t *)&id[0], &decicelsius ) == DS18X20_OK ) 
			}while(retry<3);
		}
		else 
			error++;
//-----------------------------------------------------------------
	} // while ( diff != OW_LAST_DEVICE && (a+nSensors) < MAXSENSORS )
}


void uart_put_temp(int16_t decicelsius)
{
	char s[10];
	usart_write("%i deciC, ", decicelsius );
	DS18X20_format_from_decicelsius( decicelsius, s, 10 );
	usart_write("%s C\r\n", s );
}


#if DS18X20_MAX_RESOLUTION
void uart_put_temp_maxres(int32_t tval)
{
	char s[10];
	ultoa( tval, s, 10 );
	usart_write("%s C\r\n", s );
	usart_write(" Ce-4, ");
	DS18X20_format_from_maxres( tval, s, 10 );
	usart_write("%s C\r\n", s );
}
#endif /* DS18X20_MAX_RESOLUTION */


#if DS18X20_EEPROMSUPPORT
void th_tl_dump(uint8_t *sp)
{
	DS18X20_read_scratchpad( &gSensorIDs[0][0], sp, DS18X20_SP_SIZE );
	usart_write("TH/TL in scratchpad of sensor 1 now : ");
	usart_write("%i",sp[DS18X20_TH_REG]);
	usart_write(" / "); 
	usart_write("%i", sp[DS18X20_TL_REG]);
	usart_write("\r\n"); 
}


void eeprom_test(void)
{
	uint8_t sp[DS18X20_SP_SIZE], th, tl;
	
	usart_write("DS18x20 EEPROM support test for first sensor \r\n"); 
	// DS18X20_eeprom_to_scratchpad(&gSensorIDs[0][0]); // already done at power-on
	th_tl_dump( sp );
	th = sp[DS18X20_TH_REG];
	tl = sp[DS18X20_TL_REG];
	tl++;
	th++;
	DS18X20_write_scratchpad( &gSensorIDs[0][0], th, tl, DS18B20_12_BIT );
	usart_write("TH+1 and TL+1 written to scratchpad \r\n"); 
	th_tl_dump( sp );
	DS18X20_scratchpad_to_eeprom( DS18X20_POWER_PARASITE, &gSensorIDs[0][0] );
	usart_write("scratchpad copied to DS18x20 EEPROM \r\n"); 
	DS18X20_write_scratchpad( &gSensorIDs[0][0], 0, 0, DS18B20_12_BIT );
	usart_write("TH and TL in scratchpad set to 0 \r\n"); 
	th_tl_dump( sp );
	DS18X20_eeprom_to_scratchpad(&gSensorIDs[0][0]);
	usart_write("DS18x20 EEPROM copied back to scratchpad \r\n"); 
	DS18X20_read_scratchpad( &gSensorIDs[0][0], sp, DS18X20_SP_SIZE );
	if ( ( th == sp[DS18X20_TH_REG] ) && ( tl == sp[DS18X20_TL_REG] ) ) {
		usart_write("TH and TL verified \r\n"); 
	} else {
		usart_write("verify failed \r\n"); 
	}
	th_tl_dump( sp );
}
#endif /* DS18X20_EEPROMSUPPORT */
